@@ -24,8 +24,13 @@ from .utils.selenium_core import SeleniumDriverCore
 
 class SmartDriver(SeleniumDriverCore):
     def __init__(self, driver, api_key, initialization_dict={}):
-        self.version = 'appium-0.1.21'
+        self.version = 'appium-0.1.22'
         self.root_elem = None
+        caps = driver.desired_capabilities
+        self.automation_name = caps.get('automationName', '')
+        if caps.get('platformName', '') == 'Android' and self.automation_name == '':
+            self.automation_name = 'UiAutomator2'
+        self._driver_type = 'appium'
         SeleniumDriverCore.__init__(self, driver, api_key, initialization_dict)
         self._driver_type = 'appium'
         window_size = self.driver.get_window_size()
@@ -33,6 +38,8 @@ class SmartDriver(SeleniumDriverCore):
         im = Image.open(io.BytesIO(base64.b64decode(screenshotBase64)))
         width, height = im.size
         self.multiplier = 1.0 * width / window_size['width']
+        self.is_espresso = self.automation_name.lower() == 'espresso'
+        self.is_ios = self.automation_name.lower() == 'xcuitest'
 
 
     def implicitly_wait(self, wait_time):
@@ -49,8 +56,9 @@ class SmartDriver(SeleniumDriverCore):
             element_box, needs_reload = self._test_case_get_box(element_name)
             if element_box:
                 real_elem = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier)
-                element = ai_elem(real_elem.parent, real_elem, element_box, self.driver,
-                                  self.multiplier)
+                #element = ai_elem(real_elem.parent, real_elem, element_box, self.driver,
+                #                  self.multiplier)
+                element = real_elem
                 return element, self.last_test_case_screenshot_uuid, msg
             else:
                 label_url = self.url + '/testcase/label?test_case_name=' + urllib.parse.quote(self.test_case_uuid)
@@ -62,8 +70,8 @@ class SmartDriver(SeleniumDriverCore):
                     if element_box is not None:
                         print('Element was labeled, moving on')
                         real_elem = self._match_bounding_box_to_web_element(element_box, multiplier=self.multiplier)
-                        element = ai_elem(real_elem.parent, real_elem, element_box, self.driver,
-                                          self.multiplier)
+                        element = ai_elem(real_elem.parent, real_elem._id, element_box, self.driver,
+                                          self.multiplier, smart_driver=self, base_elem=real_elem)
                         return element, self.last_test_case_screenshot_uuid, msg
                     time.sleep(2)
         else:
@@ -78,8 +86,9 @@ class SmartDriver(SeleniumDriverCore):
                 if self.debug:
                     print(f'Found cached box in action info for {element_name} using that')
                 real_elem = self._match_bounding_box_to_web_element(resp_data['box'], multiplier=self.multiplier)
-                element = ai_elem(real_elem.parent, real_elem, resp_data['box'], self.driver,
-                                  self.multiplier)
+                #element = ai_elem(real_elem.parent, real_elem, resp_data['box'], self.driver,
+                #                  self.multiplier)
+                element = real_elem
                 return element, key, msg
 
             source = ''
@@ -123,7 +132,7 @@ class SmartDriver(SeleniumDriverCore):
                         element_box, multiplier=self.multiplier)
                     parent_elem = real_elem.parent
                     elem_id = real_elem._id
-                element = ai_elem(parent_elem, elem_id, element_box, self.driver, self.multiplier)
+                element = ai_elem(parent_elem, elem_id, element_box, self.driver, self.multiplier, smart_driver=self, base_elem=real_elem)
             except Exception:
                 logging.exception('exception during classification')
             return element, run_key, msg
@@ -142,7 +151,6 @@ class SmartDriver(SeleniumDriverCore):
         try:
             elements = self.driver.find_elements(by='xpath', value='//*')
         except StaleElementReferenceException:
-            self.driver.refresh()
             elements = self.driver.find_elements(by='xpath', value='//*')
 
         # Compute IOU
@@ -160,6 +168,26 @@ class SmartDriver(SeleniumDriverCore):
         composite = filter(lambda x: x[0] > 0, composite)
         composite = list(filter(lambda x: self._center_hit(new_box, x[1].rect), composite))
 
+        attribute_for_class = "className"
+        if self.is_espresso:
+            attribute_for_class = "class"
+        elif self.is_ios:
+            attribute_for_class = "type"
+
+        interactable_elements = ["input", "edittext", "edit", "select", "dropdown", "button", "textfield", "textarea", "picker", "spinner"]
+        non_interactable_elements = ["layout"]
+
+        for score, element in composite:
+            for interactable_class in interactable_elements:
+                if interactable_class in element.get_attribute(attribute_for_class).lower() and score > 0.6 * composite[0][0]:
+                    contains_non_interactable = False
+                    for non_interactable_class in non_interactable_elements:
+                        if non_interactable_class in element.get_attribute(attribute_for_class).lower():
+                            contains_non_interactable = True
+                            break
+                    if not contains_non_interactable:
+                        return element
+
         if len(composite) == 0:
             raise NoElementFoundException('Could not find any web element under the center of the bounding box')
         else:
@@ -170,10 +198,11 @@ class SmartDriver(SeleniumDriverCore):
 
 
 class ai_elem(webdriver.webelement.WebElement):
-    def __init__(self, parent, _id, elem, driver, multiplier=1.0):
+    def __init__(self, parent, _id, elem, driver, multiplier=1.0, smart_driver=None, base_elem=None):
         super(ai_elem, self).__init__(parent, _id)
         multiplier = max(1, multiplier)
         self.driver = driver
+        self.smart_driver = smart_driver
         self._text = elem.get('text', '')
         self._size = {'width': elem.get('width', 0) / multiplier, 'height': elem.get('height', 0) / multiplier}
         self._location = {'x': elem.get('x', 0) / multiplier, 'y': elem.get('y', 0) / multiplier}
@@ -184,6 +213,7 @@ class ai_elem(webdriver.webelement.WebElement):
         self._tag_name = elem.get('class', '')
         self._cx = elem.get('x', 0) / multiplier + elem.get('width', 0) / multiplier / 2
         self._cy = elem.get('y', 0) / multiplier + elem.get('height', 0) / multiplier / 2
+        self.real_elem = base_elem
 
     @property
     def size(self):
@@ -207,9 +237,12 @@ class ai_elem(webdriver.webelement.WebElement):
     def send_keys(self, value, click_first=True):
         if click_first:
             self.click()
-        actions = ActionChains(self.driver)
-        actions.send_keys(value)
-        actions.perform()
+        if not self.smart_driver.is_ios:
+            actions = ActionChains(self.driver)
+            actions.send_keys(value)
+            actions.perform()
+        else:
+            self.real_elem.send_keys(value)
 
     def submit(self):
         self.send_keys('\n', click_first=False)
